# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: CERT.pm,v 1.10 2003/07/04 22:58:58 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CERT;

use POSIX;
use Locale::gettext;

sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};

   bless($self, $class);
}

sub read_certlist {
   my $self    = shift;
   my $main    = shift;
   my $force   = shift;

   my $ca      = $main->{'CA'}->{'actca'};
   my $certdir = $main->{'CA'}->{$ca}->{'dir'}."/certs";
   my $crlfile = $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem";

   my($f, $certlist, $crl, $modt, $parsed, $tmp);

   $certlist = [];
   
   $modt = (stat($certdir))[9];

   if(defined($self->{'lastread'}) &&
      ($self->{'lastread'} >= $modt) && 
      not defined($force)) {
      return(0);
   }

   $crl = $main->{'OpenSSL'}->parsecrl($main, $crlfile, $force);
   
   opendir(DIR, $certdir) || do {
      $main->print_warning(gettext("Can't open certdir"));
      return(0);
   };

   while($f = readdir(DIR)) {
      next if $f =~ /^\./;

      $f =~ s/\.pem//;
      
      $parsed = $self->parse_cert( $main, $f, $force);

      defined($parsed) || $main->print_error(gettext("Can't read certificate"));
      
      $tmp = MIME::Base64::decode($f);
      next if not defined($tmp);
      next if $tmp eq "";

      $tmp .= "%".$parsed->{'STATUS'};

      push(@{$certlist}, $tmp);
   }
   @{$certlist} = sort(@{$certlist});
   closedir(DIR);

   $self->{'certlist'} = $certlist;

   $self->{'lastread'} = time();
   return(1);  # got new list
}

sub revoke_cert {
   my $self = shift;
   my $main = shift;
   my $ca  = $main->{'CA'}->{'actca'};
   my $ind = $main->{'certtab'}->{'list'}->info('selection');

   my($certname, $cert, $status, $certfile, $passwd, $action, $t);

   if(not defined $ind) {
      $main->print_info(gettext("Please select a certificate first"));
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   if($status ne gettext("VALID")) {
      $t = sprintf(gettext("Can't revoke Certifikate with Status: %s"), 
            $status);
      $main->print_warning($t);
      return;
   }

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   $action = $main->show_cert_revoke_dialog(\$passwd);

   return if $action eq gettext("Cancel");

   if($main->{'OpenSSL'}->revoke(
         'config' => $main->{'CA'}->{$ca}->{'cnf'},
         'infile' => $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem",
         'pass'   => $passwd
         )) {
      $main->print_warning(gettext("Revoke failed"));
      return;
   }

   $main->{'OpenSSL'}->newcrl(
         $main,
         'config'  => $main->{'CA'}->{$ca}->{'cnf'},
         'pass'    => $passwd,
         'crldays' => 365,
         'outfile' => $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem"
         );

   if (not -s $main->{'CA'}->{$ca}->{'dir'}."/crl/crl.pem") { 
      $main->print_error(gettext("Generating CRL failed"));
   }

   $action = $main->show_key_del_question();

   if($action eq gettext("Yes")) {
      unlink($main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem");
      unlink($main->{'CA'}->{$ca}->{'dir'}."/req/".$certname.".pem");
   }

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub del_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status, $t);

   if(not defined $ind) {
      $main->print_info(gettext("Please select a certificate first"));
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";

   if($status eq gettext("VALID")) {
      $main->print_warning(gettext("Certificate is not revoked"));
      return;
   }

   unlink($certfile);

   # force reread of certlist
   $self->read_certlist($main, 1);

   $main->create_mframe();

   return;
}

sub export_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $main->{'CA'}->{'actca'};
   my $ind  = $main->{'certtab'}->{'list'}->info('selection');
    
   my($certname, $cert, $certfile, $status, $outfile, $action, 
      $format, $x509, $out, $keyfile, $passwd, $p12passwd, 
      $includeca, $parsed, $cafile, $t);

   if(not defined $ind) {
      $main->print_info(gettext("Please select a certificate first"));
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');
   $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$certname.".pem";
   $keyfile  = $main->{'CA'}->{$ca}->{'dir'}."/keys/".$certname.".pem";
   $cafile   = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";

   if($status ne gettext("VALID")) {
      $main->print_warning(gettext("Certificate seems not to be valid"));
      return;
   }

   $action = $main->show_cert_export_dialog( \$outfile, \$format);

   return if $action eq gettext("Cancel");
   return if(not defined($outfile));

   $parsed = $self->parse_cert($main, $certname);
   
   if($format eq "PEM") {
      $out = $parsed->{'PEM'};
   } elsif ($format eq "DER") {
      $out = $parsed->{'DER'};
   } elsif ($format eq "TXT") {
      $out = $parsed->{'TEXT'};
   } elsif ($format eq "P12") {

      if(not -s $keyfile) {
         $main->print_warning(gettext("Key is necessary for export as PKCS#12"));
         return;
      }

      $includeca = 0;
      $action = $main->show_p12_export_dialog(
            \$p12passwd,
            \$passwd,
            \$includeca
            );

      return if $action eq gettext("Cancel");
      
      unlink($outfile);
      $main->{'OpenSSL'}->genp12(
            main      => $main,
            certfile  => $certfile,
            keyfile   => $keyfile,
            cafile    => $cafile,
            outfile   => $outfile,
            passwd    => $passwd,
            p12passwd => $p12passwd,
            includeca => $includeca
            );

      if(not -s $outfile) {
         $main->print_warning(gettext("Generating PKCS#12 failed"));
         return;
      }

      $t = sprintf(gettext("Certificate and Key exported to %s"), $outfile);
      $main->print_info($t);
      return;

   } elsif ($format eq "ZIP") {

      if(not -s $keyfile) {
         $main->print_warning(gettext("Key is necessary for export as Zip"));
         return;
      }

      my $tmpdir    = $main->{'init'}->{'basedir'}."/tmp";
      my $tmpcert   = "$tmpdir/cert.pem";
      my $tmpkey    = "$tmpdir/key.pem";
      my $tmpcacert = "$tmpdir/cacert.pem";

      open(OUT, ">$tmpcert") || do {
         $main->print_warning(gettext("Can't create temporary file"));
         return;
      };
      print OUT $parsed->{'PEM'};
      close OUT;

      # store key in temporary location
      {
      open(IN, "<$keyfile") || do {
         $main->print_warning(gettext("Can't read key file"));
         return;
      };
      my @key = <IN>;
      close IN;

      open(OUT, ">$tmpkey") || do {
         $main->print_warning(gettext("Can't create temporary file"));
         return;
      };
      print OUT @key;
      close OUT;
      }

      # store cacert in temporary location
      {
      open(IN, "<$cafile") || do {
         $main->print_warning(gettext("Can't read CA certificate"));
         return;
      };
      my @cacert = <IN>;
      close IN;

      open(OUT, ">$tmpcacert") || do {
         $main->print_warning(gettext("Can't create temporary file"));
         return;
      };
      print OUT @cacert;
      close OUT;
      }

      unlink($outfile);
      system($main->{'init'}->{'zipbin'}, '-j', $outfile, $tmpcacert, 
             $tmpkey, $tmpcert);
      my $ret = $? >> 8;

      if(not -s $outfile || $ret) {
         $main->print_warning(gettext("Generating zip file failed"));
      } else {
         $t = sprintf(gettext("Certificate and key exported to %s"), $outfile);
         $main->print_info($t);
      unlink($tmpcacert);
      unlink($tmpcert);
      unlink($tmpkey);

      return;
      }

   } else {
      $t = sprintf(gettext("Invalid Format: %s"), $format);
      $main->print_warning($t);
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning(gettext("Can't open output file: %s: %s"),
            $outfile, $!);
      return;
   };

   print OUT $out;
   close OUT;
   
   $t = sprintf(gettext("Certificate exported to: %s"), $outfile);
   $main->print_info($t);

   return;
}

sub parse_cert {
   my $self  = shift;
   my $main  = shift;
   my $name  = shift;
   my $force = shift;

   my($ca, $certfile, $x509, $parsed);

   $ca = $main->{'CA'}->{'actca'};

   if($name eq 'CA') {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/cacert.pem";
   } else {
      $certfile = $main->{'CA'}->{$ca}->{'dir'}."/certs/".$name.".pem";
   }

   $parsed = $main->{'OpenSSL'}->parsecert(
         $main, 
         $certfile,
         $force
         );

   return($parsed);
}

1
